<?php

namespace Drupal\date_group\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\datetime_range\Plugin\Field\FieldFormatter\DateRangeDefaultFormatter;

/**
 * Plugin implementation of the 'date_group' formatter for 'daterange' fields."
 *
 * @fieldFormatter(
 *   id = "date_group",
 *   label = @Translation("Date group"),
 *   field_types = {
 *     "daterange"
 *   }
 * )
 */
class DateGroupFormatter extends DateRangeDefaultFormatter {

  /**
   * All possible formats used by a date containing day, month and year.
   */
  private $allDateFormats = [
    'day' => ['d', 'D', 'j', 'l', 'N'],
    'month' => ['F', 'm', 'M', 'n', 't'],
    'year' => ['o', 'Y', 'y'],
  ];

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    foreach ($items as $delta => $item) {
      if (!empty($item->start_date) && !empty($item->end_date)) {
        /** @var \Drupal\Core\Datetime\DrupalDateTime $start_date */
        $start_date = $item->start_date;
        /** @var \Drupal\Core\Datetime\DrupalDateTime $end_date */
        $end_date = $item->end_date;

        if ($start_date->getTimestamp() !== $end_date->getTimestamp()) {
          $group_dates = $this->groupDates($start_date, $end_date);

          $elements[$delta] = [
            '#markup' => $group_dates,
            '#cache' => [
              'contexts' => [
                'timezone',
              ],
            ],
          ];
        }
        else {
          $elements[$delta] = $this->buildDate($item->start_date);

          if (!empty($item->_attributes)) {
            $elements[$delta]['#attributes'] += $item->_attributes;
            // Unset field item attributes since they have been included in the
            // formatter output and should not be rendered in the field template.
            unset($item->_attributes);
          }
        }
      }
    }

    return $elements;
  }

  private function groupDates($start_date, $end_date) {
    $format_type = $this->getSetting('format_type');
    $date_format = \Drupal::entityTypeManager()
      ->getStorage('date_format')
      ->load($format_type);

    $start = date_parse($start_date);
    $end = date_parse($end_date);

    if ($start['year'] != $end['year']) {
      return $this->formatDateYear($start_date, $end_date);
    }

    if ($start['month'] == $end['month']) {
      return $this->formatDateMonth($start_date, $end_date, $date_format);
    }
    else {
      return $this->formatDateDay($start_date, $end_date, $date_format);
    }
  }

  private function formatDateYear($start_date, $end_date) {
    $start = $this->formatDate($start_date);
    $end = $this->formatDate($end_date);

    return $start . $this->getSeparator() . $end;
  }

  private function formatDateMonth($start_date, $end_date, $date_format) {
    $split_format = $this->getSplittedFormat($date_format);
    $start = date_parse($start_date);
    $end = date_parse($end_date);
    $collect_date = '';
    $collect_end = '';

    foreach ($split_format as $format) {
      if (in_array($format, $this->allDateFormats['day']) && $start['day'] != $end['day']) {
        $collect_date .= $format . $this->getSeparator() . '%';
        $collect_end = $format;
      }
      else {
        $collect_date .= $format;
      }
    }

    // Format different pieces of date.
    $start = $this->groupDateFormat($start_date->getTimestamp(), $collect_date);
    $end = $this->groupDateFormat($end_date->getTimestamp(), $collect_end);

    // Replace end date placeholder.
    return str_replace('%', $end, $start);
  }

  private function formatDateDay($start_date, $end_date, $date_format) {
    $separator = $this->getSeparator();
    $split_format = $this->getSplittedFormat($date_format);
    $collect_date = '';
    $collect_year = '';

    foreach ($split_format as $format) {
      if (in_array($format, $this->allDateFormats['month'])) {
        $is_month = TRUE;
        $is_date = TRUE;
        $collect_date .= $format;
      }
      elseif (in_array($format, $this->allDateFormats['day'])) {
        $is_day = TRUE;
        $is_date = TRUE;
        $collect_date .= $format;
      }
      elseif ((empty($is_month) || empty($is_day)) && !in_array($format, $this->allDateFormats['year'])) {
        if (!empty($collect_year) && !isset($is_date)) {
          $year_first = TRUE;
          $collect_year .= $format;
        }
        else {
          $collect_date .= $format;
        }
      }
      else {
        $collect_year .= $format;
      }
    }

    // Format different pieces of date.
    $start = $this->groupDateFormat($start_date->getTimestamp(), $collect_date);
    $end = $this->groupDateFormat($end_date->getTimestamp(), $collect_date);
    $year = $this->groupDateFormat($start_date->getTimestamp(), $collect_year);

    // Concatenate final date format.
    if (isset($year_first)) {
      $group_date_format = $year . $start . $separator . $end;
    }
    else {
      $group_date_format = $start . $separator . $end . $year;
    }

    return $group_date_format;
  }

  private function getSeparator() {
    return ' ' . $this->getSetting('separator') . ' ';
  }

  private function getSplittedFormat($date_format) {
    return str_split($date_format->getPattern(), 1);
  }

  private function groupDateFormat($timestamp, $format) {
    return $this->dateFormatter->format($timestamp, 'custom', $format);
  }

}
